﻿using System;
using ThinkGeo.MapSuite;
using ThinkGeo.MapSuite.Drawing;
using ThinkGeo.MapSuite.Layers;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Styles;
using ThinkGeo.MapSuite.WebForms;

namespace _9462
{
    public partial class WebForm1 : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                // Set the Map Unit. The reason for setting it to DecimalDegrees is that is what the shapefile’s unit of measure is inherently in.
                Map1.MapUnit = GeographyUnit.Meter;

                ThinkGeoCloudMapsOverlay worldOverlay = new ThinkGeoCloudMapsOverlay();
                worldOverlay.WrapDateline = WrapDatelineMode.WrapDateline;
                Map1.CustomOverlays.Add(worldOverlay);
                Map1.ZoomLevelSet = ThinkGeoCloudMapsOverlay.GetZoomLevelSet();
                LayerOverlay layerOverlay = new LayerOverlay();
                layerOverlay.TileType = TileType.SingleTile;
                Map1.CustomOverlays.Add(layerOverlay);

                InMemoryFeatureLayer lineLayer = new InMemoryFeatureLayer();
                lineLayer.WrappingMode = WrappingMode.WrapDateline;
                //lineLayer.WrappingExtent = new RectangleShape(-180, 90, 180, 90);
                lineLayer.WrappingExtent = new RectangleShape(-20000000, 20000000, 20000000, -20000000);
                lineLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.StandardColors.Red, 5, true);
                lineLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
                layerOverlay.Layers.Add(lineLayer);

                PointShape p1 = new PointShape(127.233000, 38.317000);
                PointShape p2 = new PointShape(-100.377293, 39.257254);

                lineLayer.InternalFeatures.Add(GetLine3857(GetProjection().ConvertToExternalProjection(p1) as PointShape, GetProjection().ConvertToExternalProjection(p2) as PointShape));
                lineLayer.Open();
                Map1.CurrentExtent = GetProjection().ConvertToExternalProjection(lineLayer.GetBoundingBox());
            }
        }

        private Proj4Projection GetProjection()
        {

            Proj4Projection proj = new Proj4Projection();
            proj.InternalProjectionParametersString = Proj4Projection.GetEpsgParametersString(4326);
            proj.ExternalProjectionParametersString = Proj4Projection.GetEpsgParametersString(3857);
            proj.Open();
            return proj;
        }


        private Feature GetLine(PointShape p1, PointShape p2)
        {
            PointShape westPoint;
            PointShape eastPoint;

            if (p2.X > p1.X)
            {
                westPoint = p1;
                eastPoint = p2;
            }
            else
            {
                westPoint = p2;
                eastPoint = p1;
            }


            double distanceX = eastPoint.X - westPoint.X;

            if (distanceX > 180)
            {
                MultilineShape multLine = new MultilineShape();

                LineShape line1 = new LineShape();
                line1.Vertices.Add(new Vertex(westPoint.X + 360, westPoint.Y));
                line1.Vertices.Add(new Vertex(eastPoint.X, eastPoint.Y));
                LineShape line2 = new LineShape();
                line2.Vertices.Add(new Vertex(westPoint.X, westPoint.Y));
                line2.Vertices.Add(new Vertex(eastPoint.X - 360, eastPoint.Y));

                multLine.Lines.Add(line1);
                multLine.Lines.Add(line2);

                return new Feature(multLine);
            }
            else
            {
                LineShape line = new LineShape();
                line.Vertices.Add(new Vertex(westPoint));
                line.Vertices.Add(new Vertex(eastPoint));
                return new Feature(line);
            }
        }

        private Feature GetLine3857(PointShape p1, PointShape p2)
        {
            PointShape westPoint;
            PointShape eastPoint;

            if (p2.X > p1.X)
            {
                westPoint = p1;
                eastPoint = p2;
            }
            else
            {
                westPoint = p2;
                eastPoint = p1;
            }


            double distanceX = eastPoint.X - westPoint.X;

            if (distanceX > 20000000)
            {
                MultilineShape multLine = new MultilineShape();

                LineShape line1 = new LineShape();
                line1.Vertices.Add(new Vertex(westPoint.X + 40000000, westPoint.Y));
                line1.Vertices.Add(new Vertex(eastPoint.X, eastPoint.Y));
                LineShape line2 = new LineShape();
                line2.Vertices.Add(new Vertex(westPoint.X, westPoint.Y));
                line2.Vertices.Add(new Vertex(eastPoint.X - 40000000, eastPoint.Y));

                multLine.Lines.Add(line1);
                multLine.Lines.Add(line2);

                return new Feature(multLine);
            }
            else
            {
                LineShape line = new LineShape();
                line.Vertices.Add(new Vertex(westPoint));
                line.Vertices.Add(new Vertex(eastPoint));
                return new Feature(line);
            }
        }
    }
}